from . import tools


def request_started_handler(sender, **kwargs):
    tools.set_request_id()
    tools.set_request_start_time()


def request_finished_handler(sender, **kwargs):
    tools.clear_storage()



import logging
from time import time

from . import tools


LOGGER = logging.getLogger(__name__)


class LoggingContextMiddleware:

    SKIPPED_MEDIA_TYPES = ('text/html', 'text/javascript')

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        LOGGER.debug(
            f'{request.method} {request.build_absolute_uri()} BODY:'
            f" {request.body.decode('utf-8', errors='ignore')}")
        tools.set_django_request(request)
        return self.process_response(self.get_response(request))

    def process_response(self, response):
        tools.set_django_response(response)
        request_start_time = tools.get_request_start_time()
        if request_start_time:
            tools.set_response_duration(time() - request_start_time)

        content_type = response.get('content-type')
        if content_type:
            media_type = content_type[1].split(';')[0]
            if media_type in self.SKIPPED_MEDIA_TYPES:
                return response

        result = []

        def log_streaming_content(content):
            for chunk in content:
                result.append(chunk.decode('utf-8', errors='ignore'))
                yield chunk

        if response.streaming:
            response.streaming_content = log_streaming_content(
                response.streaming_content)
        else:
            result.append(response.content.decode('utf-8', errors='ignore'))

        LOGGER.debug(f"RESPONSE: {''.join(result)}")
        return response



from django.apps import AppConfig as BaseAppConfig
from django.core.signals import request_started, request_finished

from .signals import (
    request_finished_handler, request_started_handler)


class AppConfig(BaseAppConfig):
    name = 'django_logging_context'

    def ready(self):
        request_started.connect(
            request_started_handler, dispatch_uid='request_started')
        request_finished.connect(
            request_finished_handler, dispatch_uid='request_finished')



import uuid
from time import time
from threading import current_thread


_THREAD_STORAGES = {}


USER_KEY = 'user'
REQUEST_ID_KEY = 'request_id'
DJANGO_REQUEST_KEY = 'django_request'
DJANGO_RESPONSE_KEY = 'django_response'
RESPONSE_DURATION_KEY = 'response_duration'
REQUEST_START_TIME = 'request_start_time'


def _get_thread_key():
    return current_thread()


def _get_storage():
    thread_key = _get_thread_key()
    storage = _THREAD_STORAGES.get(thread_key)
    if storage is None:
        _THREAD_STORAGES[thread_key] = storage = {}
    return storage


def _get_user():
    user = _get_storage().get(USER_KEY)
    if user:
        return user
    django_request = get_django_request()
    if django_request and hasattr(django_request, 'user'):
        return django_request.user
    return None


def get_django_request():
    return _get_storage().get(DJANGO_REQUEST_KEY)


def get_django_response():
    return _get_storage().get(DJANGO_RESPONSE_KEY)


def get_user_id():
    user = _get_user()
    if user:
        return user.id
    return None


def get_username():
    user = _get_user()
    if user:
        return user.username
    return None


def get_remote_addr():
    request = get_django_request()
    if request:
        return request.META['REMOTE_ADDR']


def set_django_request(value):
    _get_storage()[DJANGO_REQUEST_KEY] = value


def set_django_response(value):
    _get_storage()[DJANGO_RESPONSE_KEY] = value


def set_request_start_time():
    _get_storage()[REQUEST_START_TIME] = time()


def get_request_start_time():
    return _get_storage().get(REQUEST_START_TIME)


def set_response_duration(duration):
    _get_storage()[RESPONSE_DURATION_KEY] = duration


def get_response_duration():
    return _get_storage().get(RESPONSE_DURATION_KEY)


def set_request_id():
    _get_storage()[REQUEST_ID_KEY] = uuid.uuid4()


def get_request_id():
    return _get_storage().get(REQUEST_ID_KEY)


def clear_storage():
    _THREAD_STORAGES.pop(_get_thread_key(), None)



from setuptools import find_packages, setup


with open('requirements.txt') as fin:
    REQUIREMENTS = [line.strip() for line in fin if '#' not in line]


with open('README.md') as readme:
    README = readme.read()


setup(
    name='django-logging-context',
    version='1.1.1',
    packages=find_packages(),
    include_package_data=True,
    license='BSD',
    long_description_content_type="text/markdown",
    description=(
        'Django library extending default logging context'),
    long_description=README,
    install_requires=REQUIREMENTS,
    url='https://github.com/pik-software/django-logging-context',
    author='Pik Digital',
    author_email='nsi-dev@pik.ru',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: Log Analysis',
        'Topic :: System :: Logging',
    ]
)



import logging

from . import tools


LOGGER = logging.getLogger(__name__)


class ContextExtendingFilter(logging.Filter):

    def filter(self, record):
        record.request_id = tools.get_request_id()
        record.user_id = tools.get_user_id()
        record.username = tools.get_username()
        record.remote_addr = tools.get_remote_addr()
        response_duration = tools.get_response_duration()
        request = tools.get_django_request()
        record.uri = None
        if request and request.build_absolute_uri:
            record.uri = request.build_absolute_uri()
        if response_duration is not None:
            response_duration = '{0:0.6f}s'.format(response_duration)
        record.response_duration = response_duration
        return True



import logging
from django.core.servers.basehttp import ServerHandler
from . import tools


LOGGER = logging.getLogger(__name__)


class LoggingWSGIMiddleware:

    def __init__(self, application):
        self.application = application

    def __call__(self, environ, start_response):
        for chunk in self.application(environ, start_response):
            yield chunk
        if not isinstance(start_response.__self__, ServerHandler):
            request = tools.get_django_request()
            response = tools.get_django_response()
            if request and response:
                content_length = response.get('content-length')
                server_protocol = request.environ.get('SERVER_PROTOCOL')
                LOGGER.info(
                    f'"{request.method} {request.path} {server_protocol}"'
                    f' {response.status_code} {content_length}')
